@@ -3,11 +3,15 @@ package IO::Pipe::Producer;
 use 5.012003;
 use strict;
 use warnings;
+use Carp;
 
 our @ISA = qw(IO::Pipe);
 use base qw(IO::Pipe);
 
-our $VERSION = '1.6';
+our $VERSION = '1.7';
+
+our $handle_buffer       = [];   #For tracking unclosed handles
+our $max_expired_handles = 200;
 
 #NOTICE
 #
@@ -80,7 +84,7 @@ sub getSubroutineProducer
       {
 	$error = "ERROR:Producer.pm:getSubroutineProducer:A referenced " .
 	  "subroutine is required as the first argument to " .
-	    "getSubroutineProducer.\n";
+	    "getSubroutineProducer.";
 	$Producer::errstr = $error;
 	carp($error);
 	return(undef);
@@ -88,7 +92,8 @@ sub getSubroutineProducer
 
     #Create a pipe
     my $stdout_pipe = $self->SUPER::new();
-    my $stderr_pipe = $self->SUPER::new();
+    my($stderr_pipe);
+    $stderr_pipe = $self->SUPER::new() if(wantarray);
 
     #Fork off the Producer
     if(defined($pid = fork()))
@@ -101,7 +106,31 @@ sub getSubroutineProducer
 
 	    #Create a read file handle
 	    $stdout_pipe->reader();
-	    $stderr_pipe->reader();
+	    $stderr_pipe->reader() if(wantarray);
+
+	    #If expired open file handles are building up
+	    my @expired = grep {defined(fileno($_)) && eof($_)}
+	      @{$IO::Pipe::Producer::handle_buffer};
+	    if(scalar(@expired) > $IO::Pipe::Producer::max_expired_handles)
+	      {
+		$error = "WARNING:Producer.pm:getSubroutineProducer:Expired " .
+		  "file handles have exceeded the max allowed: " .
+		    "[$IO::Pipe::Producer::max_expired_handles].  Cleaning " .
+		      "them up.  Please make sure your script closes the " .
+			"file handles explicitly.";
+		$Producer::errstr = $error;
+		carp($error);
+
+		close($_) foreach(@expired);
+	      }
+
+	    #Track the file handles
+	    @{$IO::Pipe::Producer::handle_buffer} =
+	      grep {defined(fileno($_))} @{$IO::Pipe::Producer::handle_buffer};
+	    push(@{$IO::Pipe::Producer::handle_buffer},$stdout_pipe);
+	    push(@{$IO::Pipe::Producer::handle_buffer},$stderr_pipe)
+	      if(wantarray);
+
 	    #Return the read file handle to the consumer
 	    if(wantarray)
 	      {return(bless($stdout_pipe,ref($self)),
@@ -117,8 +146,8 @@ sub getSubroutineProducer
 	    #Create a write file handle for the Producer
 	    $stdout_pipe->writer();
 	    $stdout_pipe->autoflush;
-	    $stderr_pipe->writer();
-	    $stderr_pipe->autoflush;
+	    $stderr_pipe->writer()  if(defined($stderr_pipe));
+	    $stderr_pipe->autoflush if(defined($stderr_pipe));
 
 	    #Redirect standard outputs to the pipes or kill the child
 	    if(!open(STDOUT,">&",\${$stdout_pipe}))
@@ -130,7 +159,7 @@ sub getSubroutineProducer
 		$Producer::errstr = $error;
 		croak($error);
 	      }
-	    elsif(!open(STDERR,">&",\${$stderr_pipe}))
+	    elsif(defined($stderr_pipe) && !open(STDERR,">&",\${$stderr_pipe}))
 	      {
 		$error = "ERROR:Producer.pm:getSubroutineProducer:Can't " .
 		  "redirect stderr to pipe: [" .
@@ -143,9 +172,9 @@ sub getSubroutineProducer
 	    #Call the subroutine passed in (ignore it's return value)
 	    $producer_sub->(@params);
 
-	    #Close the pipes (hence it's unnecessary to do so in the consumer)
-	    $stdout_pipe->close();
-	    $stderr_pipe->close();
+	    #Close the writer pipes
+	    close($stdout_pipe);
+	    close($stderr_pipe) if(defined($stderr_pipe));
 
 	    #Successfully exiting the child process
 	    exit(0);
@@ -153,8 +182,7 @@ sub getSubroutineProducer
       }
     else
       {
-	$error = "ERROR:Producer.pm:getSubroutineProducer:fork() didn't " .
-	  "work!\n";
+	$error = "ERROR:Producer.pm:getSubroutineProducer:fork() didn't work!";
 	$Producer::errstr = $error;
 	carp($error);
 	return(undef);
@@ -178,7 +206,7 @@ IO::Pipe::Producer - Perl extension for IO::Pipe
 
 =head1 SYNOPSIS
  
-  #Example 1 (Calling a subroutine and grabbing its standard output):
+  #Example 1 (Call a subroutine & grab its standard output):
  
   use IO::Pipe::Producer;
   $obj = new IO::Pipe::Producer();
@@ -187,16 +215,15 @@ IO::Pipe::Producer - Perl extension for IO::Pipe
                                 @mysub_params);
   while(<$stdout_file_handle>)
     {print}
+  close($stdout_file_handle);
  
-  #Example 2 (Calling a subroutine and grabbing both its standard output and
-  #           standard error):
+  #Example 2 (Call a subroutine & grab its standard output & standard error):
  
   ($stdout_file_handle,$stderr_file_handle) =
     $obj->getSubroutineProducer(\&mysub,
                                 @mysub_params);
-  #Note, reading all of 1 handle and then the other could lead to blocking
-  #hangs, so it is recommended to use IO::Select when reading more than 1 file
-  #handle:
+ 
+  #It is recommended to use IO::Select when reading more than 1 file handle:
   use IO::Select;
   my $sel = new IO::Select;
   $sel->add($stdout_file_handle,$stderr_file_handle);
@@ -208,6 +235,7 @@ IO::Pipe::Producer - Perl extension for IO::Pipe
           unless(defined($line))
             {
               $sel->remove($fh);
+              close($fh);
               next;
             }
           if($fh == $stdout_file_handle)
@@ -217,57 +245,31 @@ IO::Pipe::Producer - Perl extension for IO::Pipe
         }
      }
  
-  #Example 3 (You can also get subroutine producers from the constructor):
-
-  use IO::Pipe::Producer;
-  $stdout_fh = new IO::Pipe::Producer(\&mysub,
-                                      @mysub_params);
- 
-  #OR
- 
-  use IO::Pipe::Producer;
-  ($stdout_fh,$stderr_fh) =
-   new IO::Pipe::Producer(\&mysub,
-                          @mysub_params);
- 
-  #Example 4 (You can also get system-call producers using any of these
-  #           methods):
- 
-  use IO::Pipe::Producer;
-  $obj = new IO::Pipe::Producer();
-  $stdout_fh =
-    $obj->getSystemProducer("echo \"Hello World!\"");
+  #Example 3 (Grab the standard output & standard error of a system call):
  
   use IO::Pipe::Producer;
   $obj = new IO::Pipe::Producer();
   ($stdout_fh,$stderr_fh) =
     $obj->getSystemProducer("echo \"Hello World!\"");
- 
-  use IO::Pipe::Producer;
-  $stdout_fh = new IO::Pipe::Producer(sub{system(@_)},
-                                      "echo \"Hello World!\"");
- 
-  use IO::Pipe::Producer;
-  ($stdout_fh,$stderr_fh) =
-    new IO::Pipe::Producer(sub{system(@_)},
-                           "echo \"Hello World!\"");
 
 
 =head1 ABSTRACT
 
-Producer.pm is useful for piggy-backing large data processing subroutines or system calls.  Instead of making each call serially and waiting for a return or playing with temporary files, you can create a Producer that will continuosly generate output that can be further processed right away in your script.  One benefit is immediate feedback.  It's basically a way to pipe the standard output of a forked subroutine or system call to a file handle in your parent process.
+Producer.pm is useful for chaining large data processing subroutines or system calls.  Instead of making each call serially and waiting for a return, you can create a Producer that will continuosly generate output that can be immediately processed.  You can even split up input and run subroutines in parallel.  Producer.pm is basically a way to pipe the standard output of a forked subroutine or system call to a file handle in your parent process.
 
 =head1 DESCRIPTION
 
-Producer.pm is a module that provides methods to fork off a subroutine or system call and return handles on the standard output (STDOUT and STDERR).  If you have (for example) a subroutine that processes a very large text file and performs a task on each line, but you need to perform further processing with either other subroutines or in main, normally you would have to wait until the subroutine returns to get its output (either by returning it or opening a temporary file that it produced) and continue processing.  If the subroutine prints its output to STDOUT (and STDERR) or you can edit it to do so, you can call it using a Producer so that you can use the returned handle to continuously process each line as it's "produced".  You can chain subroutines together like this by having your subroutine itself create a Producer.  This is similar to using open() to run a system call, except that with this module, you can get a handle on STDERR and use it with subroutines as well.
+Producer.pm is a module that provides methods to fork off a subroutine or system call and return handles on the standard output (STDOUT and STDERR).  If you have (for example) a subroutine that processes a very large text file and performs a task on each line, but you need to perform further processing, normally you would have to wait until the subroutine returns to get its output.  If the subroutine prints its output to STDOUT (and STDERR) or you can edit it to do so, you can call it using a Producer so that you can use the returned handle to continuously process each line as it's "produced".  You can chain subroutines together like this by having your subroutine itself create a Producer.  This is similar to using open() to run a system call, except that with this module, you can get a handle on STDERR and use it with subroutines as well.  And by dividing up your input, you can take advantage of multi-core systems and process your data in a parallel fashion.
+
+Note that the handles retuned are open file handles.  It is your job to close them once you are finished with them.
 
 =head1 NOTES
 
-This module was originally written as a simple subrotuine in a library that simply used IO::Pipe.  I decided to make it into a subclass of IO::Pipe even though it only really adds one method and a helper method (Note: The getSystemProducer method calls getSubroutineProducer) because libraries seem antiquated to me.  I'm open to better design suggestions however.  I also decided to bless the returns of all the methods because they were retuning IO::Pipe objects anyway.  The basic trick this module uses can be gleaned from the IO::Pipe documentation if you read between the lines and combine various tidbits from different sections, but to simplify it, it simply "opens" the file handle inside the IO::Pipe data structure to accept output from the output file handles (STDOUT/STDERR) as input, which is the basic definition of a pipe.
+This module was originally written as a simple subrotuine that used IO::Pipe.  It adds one method and a helper method (Note: The getSystemProducer method calls getSubroutineProducer).  It functions by opening STDOUT/STDERR as input, which is the basic definition of a pipe.  Those input file handles are what is returned.
 
 =head1 BUGS
 
-No known bugs.  Please report them to I<E<lt>robleach@buffalo.eduE<gt>> if you find any.
+No known bugs.  Please report them to I<E<lt>rleach@princeton.eduE<gt>> if you find any.
 
 =head1 SEE ALSO
 
@@ -276,7 +278,7 @@ L<IO::Select>
 
 =head1 AUTHOR
 
-Robert William Leach, E<lt>rwleach@buffalo.eduE<gt>
+Robert William Leach, E<lt>rleach@princeton.eduE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
